@@ -83,8 +83,8 @@ gegl_downscale_2x2_generic (const Babl *format,
   }
   else
   {
-    in_tmp = gegl_malloc (src_height * in_tmp_rowstride);
-    out_tmp = gegl_malloc (dst_height * out_tmp_rowstride);
+    in_tmp = gegl_scratch_alloc (src_height * in_tmp_rowstride);
+    out_tmp = gegl_scratch_alloc (dst_height * out_tmp_rowstride);
     do_free = 1;
   }
 
@@ -102,8 +102,8 @@ gegl_downscale_2x2_generic (const Babl *format,
 
   if (do_free)
    {
-     gegl_free (in_tmp);
-     gegl_free (out_tmp);
+     gegl_scratch_free (out_tmp);
+     gegl_scratch_free (in_tmp);
    }
 }
 
@@ -234,18 +234,6 @@ gegl_boxfilter_u8_nl (guchar              *dest_buf,
               src[3] = src[4] - 4;
               src[6] = src[7] - 4;
 
-              if (src[0][3] == 0 &&  /* XXX: it would be even better to not call this at all for the abyss...  */
-                  src[1][3] == 0 &&
-                  src[2][3] == 0 &&
-                  src[3][3] == 0 &&
-                  src[4][3] == 0 &&
-                  src[5][3] == 0 &&
-                  src[6][3] == 0 &&
-                  src[7][3] == 0)
-              {
-                (*(uint32_t*)(dst)) = 0;
-              }
-            else
             {
               const gfloat l = left_weight[x];
               const gfloat r = right_weight[x];
@@ -382,18 +370,6 @@ gegl_boxfilter_u8_nl_alpha (guchar              *dest_buf,
               src[3] = src[4] - 4;
               src[6] = src[7] - 4;
 
-              if (src[0][3] == 0 &&
-                  src[1][3] == 0 &&
-                  src[2][3] == 0 &&
-                  src[3][3] == 0 &&
-                  src[4][3] == 0 &&
-                  src[5][3] == 0 &&
-                  src[6][3] == 0 &&
-                  src[7][3] == 0)
-              {
-                (*(uint32_t*)(dst)) = 0;
-              }
-            else
             {
               const gfloat l = left_weight[x];
               const gfloat r = right_weight[x];
@@ -557,15 +533,6 @@ gegl_bilinear_u8_nl (guchar              *dest_buf,
        break;
      case 4:
        IMPL(4,
-         if (src[0][3] == 0 &&
-             src[1][3] == 0 &&
-             src[2][3] == 0 &&
-             src[3][3] == 0)
-         {
-            (*(uint32_t*)(dst)) = 0;
-         }
-         else
-         {
            dst[0] = BILINEAR_ROUND(
                (C(src[0][0]) * rdx + C(src[1][0]) * ldx) * rdy +
                (C(src[2][0]) * rdx + C(src[3][0]) * ldx) * dy);
@@ -578,7 +545,25 @@ gegl_bilinear_u8_nl (guchar              *dest_buf,
            dst[3] = BILINEAR_ROUND(
                (C(src[0][3]) * rdx + C(src[1][3]) * ldx) * rdy +
                (C(src[2][3]) * rdx + C(src[3][3]) * ldx) * dy);
-         }
+           );
+       break;
+     case 5:
+       IMPL(5,
+           dst[0] = BILINEAR_ROUND(
+               (C(src[0][0]) * rdx + C(src[1][0]) * ldx) * rdy +
+               (C(src[2][0]) * rdx + C(src[3][0]) * ldx) * dy);
+           dst[1] = BILINEAR_ROUND(
+               (C(src[0][1]) * rdx + C(src[1][1]) * ldx) * rdy +
+               (C(src[2][1]) * rdx + C(src[3][1]) * ldx) * dy);
+           dst[2] = BILINEAR_ROUND(
+               (C(src[0][2]) * rdx + C(src[1][2]) * ldx) * rdy +
+               (C(src[2][2]) * rdx + C(src[3][2]) * ldx) * dy);
+           dst[3] = BILINEAR_ROUND(
+               (C(src[0][3]) * rdx + C(src[1][3]) * ldx) * rdy +
+               (C(src[2][3]) * rdx + C(src[3][3]) * ldx) * dy);
+           dst[4] = BILINEAR_ROUND(
+               (C(src[0][4]) * rdx + C(src[1][4]) * ldx) * rdy +
+               (C(src[2][4]) * rdx + C(src[3][4]) * ldx) * dy);
            );
        break;
 #endif
@@ -639,15 +624,6 @@ gegl_bilinear_u8_nl_alpha (guchar              *dest_buf,
        break;
      case 4:
        IMPL(4,
-         if (src[0][3] == 0 &&
-             src[1][3] == 0 &&
-             src[2][3] == 0 &&
-             src[3][3] == 0)
-         {
-            (*(uint32_t*)(dst)) = 0;
-         }
-         else
-         {
            dst[0] = BILINEAR_ROUND(
                (C(src[0][0]) * rdx + C(src[1][0]) * ldx) * rdy +
                (C(src[2][0]) * rdx + C(src[3][0]) * ldx) * dy);
@@ -660,7 +636,25 @@ gegl_bilinear_u8_nl_alpha (guchar              *dest_buf,
            dst[3] = BILINEAR_ROUND_ALPHA(
                ((src[0][3]) * rdx + (src[1][3]) * ldx) * rdy +
                ((src[2][3]) * rdx + (src[3][3]) * ldx) * dy);
-         }
+           );
+       break;
+     case 5:
+       IMPL(5,
+           dst[0] = BILINEAR_ROUND(
+               (C(src[0][0]) * rdx + C(src[1][0]) * ldx) * rdy +
+               (C(src[2][0]) * rdx + C(src[3][0]) * ldx) * dy);
+           dst[1] = BILINEAR_ROUND(
+               (C(src[0][1]) * rdx + C(src[1][1]) * ldx) * rdy +
+               (C(src[2][1]) * rdx + C(src[3][1]) * ldx) * dy);
+           dst[2] = BILINEAR_ROUND(
+               (C(src[0][2]) * rdx + C(src[1][2]) * ldx) * rdy +
+               (C(src[2][2]) * rdx + C(src[3][2]) * ldx) * dy);
+           dst[3] = BILINEAR_ROUND(
+               ((src[0][3]) * rdx + (src[1][3]) * ldx) * rdy +
+               ((src[2][3]) * rdx + (src[3][3]) * ldx) * dy);
+           dst[4] = BILINEAR_ROUND_ALPHA(
+               ((src[0][4]) * rdx + (src[1][4]) * ldx) * rdy +
+               ((src[2][4]) * rdx + (src[3][4]) * ldx) * dy);
            );
        break;
    }
@@ -945,8 +939,10 @@ GeglDownscale2x2Fun gegl_downscale_2x2_get_fun (const Babl *format)
 {
   const Babl *comp_type = babl_format_get_type (format, 0);
   const Babl *model     = babl_format_get_model (format);
-
-  if (gegl_babl_model_is_linear (model))
+  BablModelFlag model_flags = babl_get_model_flags (model);
+  
+  if ((model_flags & BABL_MODEL_FLAG_LINEAR)||
+      (model_flags & BABL_MODEL_FLAG_CMYK))
   {
     if (comp_type == gegl_babl_float())
     {
@@ -1042,8 +1038,8 @@ gegl_resample_boxfilter_generic (guchar       *dest_buf,
   }
   else
   {
-    in_tmp  = gegl_malloc (src_rect->height * in_tmp_rowstride);
-    out_tmp = gegl_malloc (dst_rect->height * out_tmp_rowstride);
+    in_tmp  = gegl_scratch_alloc (src_rect->height * in_tmp_rowstride);
+    out_tmp = gegl_scratch_alloc (dst_rect->height * out_tmp_rowstride);
     do_free = 1;
   }
 
@@ -1062,8 +1058,8 @@ gegl_resample_boxfilter_generic (guchar       *dest_buf,
 
   if (do_free)
     {
-      gegl_free (in_tmp);
-      gegl_free (out_tmp);
+      gegl_scratch_free (out_tmp);
+      gegl_scratch_free (in_tmp);
     }
 }
 
@@ -1079,8 +1075,10 @@ void gegl_resample_boxfilter (guchar              *dest_buf,
   const Babl *model     = babl_format_get_model (format);
   const Babl *comp_type  = babl_format_get_type (format, 0);
   const gint bpp = babl_format_get_bytes_per_pixel (format);
+  BablModelFlag model_flags = babl_get_model_flags (model);
 
-  if (gegl_babl_model_is_linear (model))
+  if ((model_flags & BABL_MODEL_FLAG_LINEAR)||
+      (model_flags & BABL_MODEL_FLAG_CMYK))
   {
 
     if (comp_type == gegl_babl_float())
@@ -1150,8 +1148,8 @@ gegl_resample_bilinear_generic (guchar              *dest_buf,
   }
   else
   {
-    in_tmp  = gegl_malloc (src_rect->height * in_tmp_rowstride);
-    out_tmp = gegl_malloc (dst_rect->height * out_tmp_rowstride);
+    in_tmp  = gegl_scratch_alloc (src_rect->height * in_tmp_rowstride);
+    out_tmp = gegl_scratch_alloc (dst_rect->height * out_tmp_rowstride);
     do_free = 1;
   }
 
@@ -1170,8 +1168,8 @@ gegl_resample_bilinear_generic (guchar              *dest_buf,
 
   if (do_free)
     {
-      gegl_free (in_tmp);
-      gegl_free (out_tmp);
+      gegl_scratch_free (out_tmp);
+      gegl_scratch_free (in_tmp);
     }
 }
 
@@ -1186,7 +1184,10 @@ void gegl_resample_bilinear (guchar              *dest_buf,
 {
   const Babl *model     = babl_format_get_model (format);
   const Babl *comp_type  = babl_format_get_type (format, 0);
-  if (gegl_babl_model_is_linear (model))
+  BablModelFlag model_flags = babl_get_model_flags (model);
+
+  if ((model_flags & BABL_MODEL_FLAG_LINEAR)||
+      (model_flags & BABL_MODEL_FLAG_CMYK))
   {
     const gint bpp = babl_format_get_bytes_per_pixel (format);
 
@@ -1353,8 +1354,8 @@ gegl_resample_nearest (guchar              *dst,
 
 static inline guint32 _gegl_trunc_u32(guint64 value)
 {
-  if ((guint64) value > ((1l<<32)-1))
-   return (1l<<32)-1;
+  if ((guint64) value > G_MAXUINT32)
+    return G_MAXUINT32;
   return value;
 }
 
