@@ -30,6 +30,20 @@
 
 #include "opencl/gegl-cl.h"
 
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+
+#ifdef G_OS_WIN32
+#include <windows.h>
+#endif
+
+#ifdef __APPLE__
+#include <mach/mach.h>
+#include <sys/types.h>
+#include <sys/sysctl.h>
+#endif
+
 G_DEFINE_TYPE (GeglConfig, gegl_config, G_TYPE_OBJECT)
 
 static GObjectClass * parent_class = NULL;
@@ -41,12 +55,14 @@ enum
   PROP_TILE_CACHE_SIZE,
   PROP_CHUNK_SIZE,
   PROP_SWAP,
+  PROP_SWAP_COMPRESSION,
   PROP_TILE_WIDTH,
   PROP_TILE_HEIGHT,
   PROP_THREADS,
   PROP_USE_OPENCL,
   PROP_QUEUE_SIZE,
-  PROP_APPLICATION_LICENSE
+  PROP_APPLICATION_LICENSE,
+  PROP_MIPMAP_RENDERING
 };
 
 gint _gegl_threads = 1;
@@ -85,6 +101,10 @@ gegl_config_get_property (GObject    *gobject,
         g_value_set_string (value, config->swap);
         break;
 
+      case PROP_SWAP_COMPRESSION:
+        g_value_set_string (value, config->swap_compression);
+        break;
+
       case PROP_THREADS:
         g_value_set_int (value, _gegl_threads);
         break;
@@ -101,6 +121,10 @@ gegl_config_get_property (GObject    *gobject,
         g_value_set_string (value, config->application_license);
         break;
 
+      case PROP_MIPMAP_RENDERING:
+        g_value_set_boolean (value, config->mipmap_rendering);
+        break;
+
       default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, property_id, pspec);
         break;
@@ -136,12 +160,19 @@ gegl_config_set_property (GObject      *gobject,
         g_free (config->swap);
         config->swap = g_value_dup_string (value);
         break;
+      case PROP_SWAP_COMPRESSION:
+        g_free (config->swap_compression);
+        config->swap_compression = g_value_dup_string (value);
+        break;
       case PROP_THREADS:
         _gegl_threads = g_value_get_int (value);
         return;
       case PROP_USE_OPENCL:
         config->use_opencl = g_value_get_boolean (value);
         break;
+      case PROP_MIPMAP_RENDERING:
+        config->mipmap_rendering = g_value_get_boolean (value);
+        break;
       case PROP_QUEUE_SIZE:
         config->queue_size = g_value_get_int (value);
         break;
@@ -161,6 +192,7 @@ gegl_config_finalize (GObject *gobject)
   GeglConfig *config = GEGL_CONFIG (gobject);
 
   g_free (config->swap);
+  g_free (config->swap_compression);
   g_free (config->application_license);
 
   G_OBJECT_CLASS (gegl_config_parent_class)->finalize (gobject);
@@ -183,30 +215,95 @@ gegl_config_class_init (GeglConfigClass *klass)
                                                      "Tile width",
                                                      "default tile width for created buffers.",
                                                      0, G_MAXINT, 128,
-                                                     G_PARAM_READWRITE |
-                                                     G_PARAM_CONSTRUCT));
+                                                     G_PARAM_READWRITE));
 
   g_object_class_install_property (gobject_class, PROP_TILE_HEIGHT,
                                    g_param_spec_int ("tile-height",
                                                      "Tile height",
                                                      "default tile height for created buffers.",
                                                      0, G_MAXINT, 128,
-                                                     G_PARAM_READWRITE |
-                                                     G_PARAM_CONSTRUCT));
+                                                     G_PARAM_READWRITE));
+
+  {
+    long default_tile_cache_size = 1024l * 1024 * 1024;
+    long mem_total = default_tile_cache_size;
+    long mem_min = 512 << 20; // 512mb
+    long mem_available = mem_min;
+
+#ifdef G_OS_WIN32
+# if defined(_MSC_VER) && (_MSC_VER <= 1200)
+  MEMORYSTATUS memory_status;
+  memory_status.dwLength = sizeof (memory_status);
+
+  GlobalMemoryStatus (&memory_status);
+  mem_total = memory_status.dwTotalPhys;
+  mem_available = memory_status.dwAvailPhys;
+# else
+  /* requires w2k and newer SDK than provided with msvc6 */
+  MEMORYSTATUSEX memory_status;
+
+  memory_status.dwLength = sizeof (memory_status);
+
+  if (GlobalMemoryStatusEx (&memory_status))
+  {
+    mem_total = memory_status.ullTotalPhys;
+    mem_available = memory_status.ullAvailPhys;
+  }
+# endif
+#elif defined(__APPLE__)
+    /* get total memory from the HW_MEMSIZE */
+    int64_t usermem;
+    size_t len = sizeof usermem;
+    static int mib[2] = { CTL_HW, HW_MEMSIZE };
+
+    if (sysctl (mib, 2, &usermem, &len, NULL, 0) == 0) {
+      mem_total = usermem;
+    }
+    /* and available mem from host_statistics64 */
+    vm_size_t              page_size = sysconf (_SC_PAGESIZE);
+    mach_port_t            host = mach_host_self ();
+    vm_statistics64_data_t vm_stat;
+    kern_return_t	         kret;
+    unsigned int           count = HOST_VM_INFO64_COUNT;
+
+    kret = host_statistics64 (host, HOST_VM_INFO64, (host_info64_t)&vm_stat, &count);
+
+    if (kret == KERN_SUCCESS)
+    {
+      mem_available = (long) (vm_stat.free_count + vm_stat.active_count
+                              + vm_stat.inactive_count
+# ifdef MAC_OS_X_VERSION_10_9
+                              + vm_stat.compressor_page_count
+# endif
+                             ) * page_size;
+      mach_port_deallocate (mach_task_self (), host);
+    }
+#else
+    mem_total = sysconf (_SC_PHYS_PAGES) * sysconf (_SC_PAGESIZE);
+    mem_available = sysconf (_SC_AVPHYS_PAGES) * sysconf (_SC_PAGESIZE);
+#endif
+
+    default_tile_cache_size = mem_total;
+    if (default_tile_cache_size > mem_available)
+    {
+      default_tile_cache_size = mem_available;
+    }
+    if (default_tile_cache_size < mem_min)
+      default_tile_cache_size = mem_min;
 
   g_object_class_install_property (gobject_class, PROP_TILE_CACHE_SIZE,
                                    g_param_spec_uint64 ("tile-cache-size",
                                                         "Tile Cache size",
                                                         "size of tile cache in bytes",
-                                                        0, G_MAXUINT64, 512 * 1024 * 1024,
-                                                        G_PARAM_READWRITE |
-                                                        G_PARAM_CONSTRUCT));
+                                                        0, G_MAXUINT64, default_tile_cache_size,
+                                                        G_PARAM_READWRITE));
+  }
 
   g_object_class_install_property (gobject_class, PROP_CHUNK_SIZE,
                                    g_param_spec_int ("chunk-size",
                                                      "Chunk size",
                                                      "the number of pixels processed simultaneously by GEGL.",
-                                                     1, G_MAXINT, 512 * 1024,
+                                                     1, G_MAXINT, 1024 * 1024,
                                                      G_PARAM_READWRITE |
                                                      G_PARAM_CONSTRUCT));
 
@@ -223,8 +320,14 @@ gegl_config_class_init (GeglConfigClass *klass)
                                                         "Swap",
                                                         "where gegl stores it's swap files",
                                                         NULL,
-                                                        G_PARAM_READWRITE |
-                                                        G_PARAM_CONSTRUCT));
+                                                        G_PARAM_READWRITE));
+
+  g_object_class_install_property (gobject_class, PROP_SWAP_COMPRESSION,
+                                   g_param_spec_string ("swap-compression",
+                                                        "Swap compression",
+                                                        "compression algorithm used for data stored in the swap",
+                                                        NULL,
+                                                        G_PARAM_READWRITE));
 
   _gegl_threads = g_get_num_processors ();
   _gegl_threads = MIN (_gegl_threads, GEGL_MAX_THREADS);
@@ -237,6 +340,14 @@ gegl_config_class_init (GeglConfigClass *klass)
                                                      G_PARAM_READWRITE |
                                                      G_PARAM_CONSTRUCT));
 
+  g_object_class_install_property (gobject_class, PROP_MIPMAP_RENDERING,
+                                   g_param_spec_boolean ("mipmap-rendering",
+                                                         "mipmap rendering",
+                                                         "Enable code paths for mipmap preview rendering, uses approximations for 50% 25% etc zoom factors to reduce processing.",
+                                                         FALSE,
+                                                         G_PARAM_READWRITE |
+                                                         G_PARAM_CONSTRUCT));
+
   g_object_class_install_property (gobject_class, PROP_USE_OPENCL,
                                    g_param_spec_boolean ("use-opencl",
                                                          "Use OpenCL",
@@ -250,8 +361,7 @@ gegl_config_class_init (GeglConfigClass *klass)
                                                      "Queue size",
                                                      "Maximum size of a file backend's writer thread queue (in bytes)",
                                                      2, G_MAXINT, 50 * 1024 *1024,
-                                                     G_PARAM_READWRITE |
-                                                     G_PARAM_CONSTRUCT));
+                                                     G_PARAM_READWRITE));
 
   g_object_class_install_property (gobject_class, PROP_APPLICATION_LICENSE,
                                    g_param_spec_string ("application-license",
@@ -266,6 +376,7 @@ static void
 gegl_config_init (GeglConfig *self)
 {
   char *forward_props[]={"swap",
+                         "swap-compression",
                          "queue-size",
                          "tile-width",
                          "tile-height",
@@ -274,7 +385,8 @@ gegl_config_init (GeglConfig *self)
   GeglBufferConfig *bconf = gegl_buffer_config ();
   for (int i = 0; forward_props[i]; i++)
     g_object_bind_property (bconf, forward_props[i],
-                            self, forward_props[i], G_BINDING_BIDIRECTIONAL);
+                            self, forward_props[i],
+                            G_BINDING_BIDIRECTIONAL | G_BINDING_SYNC_CREATE);
 }
 
 #undef  gegl_config_threads
