@@ -32,9 +32,6 @@
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
-#ifdef G_OS_WIN32
-#include <process.h>
-#endif
 
 #ifdef G_OS_WIN32
 
@@ -267,7 +264,8 @@ gegl_get_option_group (void)
   return group;
 }
 
-static void gegl_config_parse_env (GeglConfig *config)
+static void 
+gegl_config_parse_env (GeglConfig *config)
 {
   if (g_getenv ("GEGL_MIPMAP_RENDERING"))
     {
@@ -338,16 +336,19 @@ static void gegl_config_parse_env (GeglConfig *config)
       const char *opencl_env = g_getenv ("GEGL_USE_OPENCL");
 
       if (g_ascii_strcasecmp (opencl_env, "yes") == 0)
-        ;
+        g_object_set (config, "use-opencl", TRUE, NULL);
       else if (g_ascii_strcasecmp (opencl_env, "no") == 0)
         gegl_cl_hard_disable ();
-      else if (g_ascii_strcasecmp (opencl_env, "cpu") == 0)
+      else if (g_ascii_strcasecmp (opencl_env, "cpu") == 0) {
         gegl_cl_set_default_device_type (CL_DEVICE_TYPE_CPU);
-      else if (g_ascii_strcasecmp (opencl_env, "gpu") == 0)
+        g_object_set (config, "use-opencl", TRUE, NULL);
+      } else if (g_ascii_strcasecmp (opencl_env, "gpu") == 0) {
         gegl_cl_set_default_device_type (CL_DEVICE_TYPE_GPU);
-      else if (g_ascii_strcasecmp (opencl_env, "accelerator") == 0)
+        g_object_set (config, "use-opencl", TRUE, NULL);
+      } else if (g_ascii_strcasecmp (opencl_env, "accelerator") == 0) {
         gegl_cl_set_default_device_type (CL_DEVICE_TYPE_ACCELERATOR);
-      else
+        g_object_set (config, "use-opencl", TRUE, NULL);
+      } else
         g_warning ("Unknown value for GEGL_USE_OPENCL: %s", opencl_env);
     }
 
@@ -362,7 +363,8 @@ static void gegl_config_parse_env (GeglConfig *config)
     }
 }
 
-GeglConfig *gegl_config (void)
+GeglConfig *
+gegl_config (void)
 {
   if (!config)
     config = g_object_new (GEGL_TYPE_CONFIG, NULL);
@@ -370,7 +372,8 @@ GeglConfig *gegl_config (void)
   return config;
 }
 
-GeglStats *gegl_stats (void)
+GeglStats *
+gegl_stats (void)
 {
   if (! stats)
     stats = g_object_new (GEGL_TYPE_STATS, NULL);
@@ -378,12 +381,14 @@ GeglStats *gegl_stats (void)
   return stats;
 }
 
-void gegl_reset_stats (void)
+void 
+gegl_reset_stats (void)
 {
   gegl_stats_reset (gegl_stats ());
 }
 
-void gegl_temp_buffer_free (void);
+void 
+gegl_temp_buffer_free (void);
 
 void
 gegl_exit (void)
