@@ -11,7 +11,7 @@
  * more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with GEGL; if not, see <http://www.gnu.org/licenses/>.
+ * License along with GEGL; if not, see <https://www.gnu.org/licenses/>.
  *
  * Copyright 2016, 2017 Øyvind Kolås
  */
@@ -82,6 +82,113 @@ static void each_knot_rel (const GeglPathItem *path_node,
   g_string_append_printf (str, "%srel ", fstr);
 }
 
+
+void
+gegl_node_set_time (GeglNode   *node,
+                    gdouble     time)
+{
+  GeglNode *iter = NULL;
+  if (!node)
+    return;
+
+  if (gegl_node_has_pad (node, "input"))
+  {
+    iter = gegl_node_get_producer (node, "input", NULL);
+    if (iter)
+      gegl_node_set_time (iter, time);
+  }
+  if (gegl_node_has_pad (node, "aux"))
+  {
+    iter = gegl_node_get_producer (node, "aux", NULL);
+    if (iter)
+      gegl_node_set_time (iter, time);
+  }
+
+  {
+    gint i;
+    guint n_properties;
+    GParamSpec **properties;
+
+    properties = gegl_operation_list_properties (gegl_node_get_operation (
+                                                   node),
+                                                 &n_properties);
+    for (i = 0; i < n_properties; i++)
+      {
+        const gchar *property_name = g_param_spec_get_name (
+          properties[i]);
+        GType property_type = G_PARAM_SPEC_VALUE_TYPE (properties[i]);
+        char tmpbuf[1024];
+        GeglPath *anim_path = NULL;
+        GQuark anim_quark;//, rel_quark;
+        sprintf (tmpbuf, "%s-anim", property_name);
+        anim_quark = g_quark_from_string (tmpbuf);
+        anim_path = g_object_get_qdata (G_OBJECT (node), anim_quark);
+
+        if (property_type == G_TYPE_FLOAT)
+          {
+            if (anim_path)
+            {
+              gdouble y;
+              gegl_path_calc_y_for_x (anim_path, time, &y);
+              gegl_node_set (node, property_name, y, NULL);
+            }
+          }
+        else if (property_type == G_TYPE_DOUBLE)
+          {
+            if (anim_path)
+            {
+              gdouble y;
+              gegl_path_calc_y_for_x (anim_path, time, &y);
+              gegl_node_set (node, property_name, y, NULL);
+            }
+          }
+        else if (property_type == G_TYPE_INT)
+          {
+            if (anim_path)
+            {
+              gdouble y;
+              gegl_path_calc_y_for_x (anim_path, time, &y);
+              gegl_node_set (node, property_name, (int)y, NULL);
+            }
+          }
+        else if (property_type == G_TYPE_UINT)
+          {
+            if (anim_path)
+            {
+              gdouble y;
+              gegl_path_calc_y_for_x (anim_path, time, &y);
+              gegl_node_set (node, property_name, (guint32)y, NULL);
+            }
+          }
+        else if (property_type == G_TYPE_BOOLEAN)
+          {
+          }
+        else if (property_type == G_TYPE_STRING)
+          {
+          }
+        else if (g_type_is_a (property_type, G_TYPE_ENUM))
+          {
+          }
+        else if (property_type == GEGL_TYPE_COLOR)
+          {
+          }
+        else if (property_type == GEGL_TYPE_PATH)
+          {
+          }
+        else if (property_type == G_TYPE_POINTER &&
+                 GEGL_IS_PARAM_SPEC_FORMAT (properties[i]))
+          {
+          }
+        else
+          {
+          }
+
+      }
+    //g_free (properties);
+  }
+}
+
+
 void
 gegl_create_chain_argv (char      **ops,
                         GeglNode   *start,
@@ -103,6 +210,14 @@ gegl_create_chain_argv (char      **ops,
   GHashTable *ht = NULL;
   GeglPath   *path = NULL;
   GString    *string = NULL;
+  GeglNode **ret_sinkp = NULL;
+
+  if (error && *error)
+  {
+    GeglNode **an = (void*)error;
+    ret_sinkp = (void*)*an;
+    *error = NULL;
+  }
 
   remove_in_betweens (start, proxy);
 
@@ -110,6 +225,8 @@ gegl_create_chain_argv (char      **ops,
 
   ht = g_hash_table_new (g_str_hash, g_str_equal);
 
+
+
   while (*arg)
     {
       if (in_keyframes)
@@ -541,7 +658,8 @@ gegl_create_chain_argv (char      **ops,
                                 gegl_node_set (iter[level], key, absolute_path, NULL);
                                 free (absolute_path);
                               }
-                            gegl_node_set (iter[level], key, value, NULL);
+                            else
+                              gegl_node_set (iter[level], key, value, NULL);
                           }
                       }
                     else if (g_type_is_a (target_type, G_TYPE_STRING))
@@ -627,10 +745,17 @@ gegl_create_chain_argv (char      **ops,
                                                proxy), "operation",
                                              level_op[level], NULL);
 
-                  if (iter[level] && gegl_node_has_pad (new, "input"))
-                    gegl_node_link_many (iter[level], new, proxy, NULL);
+                  if (gegl_node_has_pad (new, "output"))
+                  {
+                    if (iter[level] && gegl_node_has_pad (new, "input"))
+                      gegl_node_link_many (iter[level], new, proxy, NULL);
+                    else
+                      gegl_node_link_many (new, proxy, NULL);
+                  }
                   else
-                    gegl_node_link_many (new, proxy, NULL);
+                  {
+                    gegl_node_link_many (iter[level], new, NULL);
+                  }
                   iter[level] = new;
                 }
               else if (error)
@@ -695,7 +820,17 @@ gegl_create_chain_argv (char      **ops,
 
   g_free (prop);
   g_hash_table_unref (ht);
-  gegl_node_link_many (iter[level], proxy, NULL);
+
+
+  if (gegl_node_has_pad (iter[level], "output"))
+    gegl_node_link_many (iter[level], proxy, NULL);
+  else
+  {
+    if (ret_sinkp)
+    {
+      *ret_sinkp = iter[level];
+    }
+  }
 }
 
 void
@@ -717,8 +852,11 @@ gegl_create_chain (const char *str, GeglNode *op_start, GeglNode *op_end,
 
 /* TODO: serialize keyframed properties */
 static gchar *
-gegl_serialize2 (GeglNode *start, GeglNode *end, const char *basepath,
-                 GHashTable *ht, GeglSerializeFlag flags)
+gegl_serialize2 (GeglNode         *start, 
+                 GeglNode         *end, 
+                 const char       *basepath,
+                 GHashTable       *ht, 
+                 GeglSerializeFlag flags)
 {
   char *ret = NULL;
   gboolean trim_defaults = flags & GEGL_SERIALIZE_TRIM_DEFAULTS;
@@ -809,30 +947,30 @@ gegl_serialize2 (GeglNode *start, GeglNode *end, const char *basepath,
                 const GValue*default_value = g_param_spec_get_default_value (
                   properties[i]);
                 char tmpbuf[1024];
-                GeglPath *path = NULL;
+                GeglPath *anim_path = NULL;
                 gboolean printed = FALSE;
                 char *rel_orig = NULL;
                 GQuark anim_quark, rel_quark;
-                sprintf (tmpbuf, "%s-anim", properties[i]->name);
+                sprintf (tmpbuf, "%s-anim", property_name);
                 anim_quark = g_quark_from_string (tmpbuf);
-                sprintf (tmpbuf, "%s-rel", properties[i]->name);
+                sprintf (tmpbuf, "%s-rel", property_name);
                 rel_quark = g_quark_from_string (tmpbuf);
-                path = g_object_get_qdata (G_OBJECT (iter), anim_quark);
+                anim_path = g_object_get_qdata (G_OBJECT (iter), anim_quark);
                 rel_orig = g_object_get_qdata (G_OBJECT (iter), rel_quark);
 
                 if (property_type == G_TYPE_FLOAT)
                   {
                     gfloat defval = g_value_get_float (default_value);
                     gfloat value;
-                    gegl_node_get (iter, properties[i]->name, &value, NULL);
+                    gegl_node_get (iter, property_name, &value, NULL);
 
-                    if (path)
+                    if (anim_path)
                     {
                       g_string_append_printf (s2, " %s={ ", property_name);
                       if (rel_orig)
-                        gegl_path_foreach (path, each_knot_rel, s2);
+                        gegl_path_foreach (anim_path, each_knot_rel, s2);
                       else
-                        gegl_path_foreach (path, each_knot, s2);
+                        gegl_path_foreach (anim_path, each_knot, s2);
                       g_string_append_printf (s2, " } ");
                     }
                     else if (value != defval || (!trim_defaults))
@@ -852,13 +990,13 @@ gegl_serialize2 (GeglNode *start, GeglNode *end, const char *basepath,
                     gdouble value;
                     gegl_node_get (iter, property_name, &value, NULL);
 
-                    if (path)
+                    if (anim_path)
                     {
                       g_string_append_printf (s2, " %s={ ", property_name);
                       if (rel_orig)
-                        gegl_path_foreach (path, each_knot_rel, s2);
+                        gegl_path_foreach (anim_path, each_knot_rel, s2);
                       else
-                        gegl_path_foreach (path, each_knot, s2);
+                        gegl_path_foreach (anim_path, each_knot, s2);
                       g_string_append_printf (s2, " } ");
                     }
                     else if (value != defval || (!trim_defaults))
@@ -877,15 +1015,15 @@ gegl_serialize2 (GeglNode *start, GeglNode *end, const char *basepath,
                     gint defval = g_value_get_int (default_value);
                     gint value;
                     gchar str[64];
-                    gegl_node_get (iter, properties[i]->name, &value, NULL);
+                    gegl_node_get (iter, property_name, &value, NULL);
 
-                    if (path)
+                    if (anim_path)
                     {
                       g_string_append_printf (s2, " %s={ ", property_name);
                       if (rel_orig)
-                        gegl_path_foreach (path, each_knot_rel, s2);
+                        gegl_path_foreach (anim_path, each_knot_rel, s2);
                       else
-                        gegl_path_foreach (path, each_knot, s2);
+                        gegl_path_foreach (anim_path, each_knot, s2);
                       g_string_append_printf (s2, " } ");
                     }
                     else if (value != defval || (!trim_defaults))
@@ -903,15 +1041,15 @@ gegl_serialize2 (GeglNode *start, GeglNode *end, const char *basepath,
                     guint defval = g_value_get_uint (default_value);
                     guint value;
                     gchar str[64];
-                    gegl_node_get (iter, properties[i]->name, &value, NULL);
+                    gegl_node_get (iter, property_name, &value, NULL);
 
-                    if (path)
+                    if (anim_path)
                     {
                       g_string_append_printf (s2, " %s={ ", property_name);
                       if (rel_orig)
-                        gegl_path_foreach (path, each_knot_rel, s2);
+                        gegl_path_foreach (anim_path, each_knot_rel, s2);
                       else
-                        gegl_path_foreach (path, each_knot, s2);
+                        gegl_path_foreach (anim_path, each_knot, s2);
                       g_string_append_printf (s2, " } ");
                     }
                     else if (value != defval || (!trim_defaults))
@@ -928,7 +1066,7 @@ gegl_serialize2 (GeglNode *start, GeglNode *end, const char *basepath,
                   {
                     gboolean value;
                     gboolean defval = g_value_get_boolean (default_value);
-                    gegl_node_get (iter, properties[i]->name, &value, NULL);
+                    gegl_node_get (iter, property_name, &value, NULL);
                     if (value != defval || (!trim_defaults))
                       {
                         if (flags & GEGL_SERIALIZE_INDENT)
@@ -946,7 +1084,7 @@ gegl_serialize2 (GeglNode *start, GeglNode *end, const char *basepath,
                   {
                     gchar *value;
                     const gchar *defval = g_value_get_string (default_value);
-                    gegl_node_get (iter, properties[i]->name, &value, NULL);
+                    gegl_node_get (iter, property_name, &value, NULL);
                     if (!g_str_equal (defval, value) || (!trim_defaults))
                       {
                         if (flags & GEGL_SERIALIZE_INDENT)
@@ -963,7 +1101,7 @@ gegl_serialize2 (GeglNode *start, GeglNode *end, const char *basepath,
                     gint defval = g_value_get_enum (default_value);
                     gint value;
 
-                    gegl_node_get (iter, properties[i]->name, &value, NULL);
+                    gegl_node_get (iter, property_name, &value, NULL);
                     if (value != defval || (!trim_defaults))
                       {
                         GEnumValue *evalue = g_enum_get_value (eclass, value);
@@ -980,7 +1118,7 @@ gegl_serialize2 (GeglNode *start, GeglNode *end, const char *basepath,
                     GeglColor *defcolor = g_value_get_object (default_value);
                     gchar     *value;
                     gchar     *defvalue = NULL;
-                    gegl_node_get (iter, properties[i]->name, &color, NULL);
+                    gegl_node_get (iter, property_name, &color, NULL);
                     g_object_get (color, "string", &value, NULL);
                     if (defcolor)
                       {
@@ -1002,7 +1140,7 @@ gegl_serialize2 (GeglNode *start, GeglNode *end, const char *basepath,
                   {
                     gchar *svg_path;
                     GeglPath *path;
-                    gegl_node_get (iter, properties[i]->name, &path, NULL);
+                    gegl_node_get (iter, property_name, &path, NULL);
                     svg_path = gegl_path_to_string (path);
                     g_object_unref (path);
                     if (flags & GEGL_SERIALIZE_INDENT)
@@ -1017,7 +1155,7 @@ gegl_serialize2 (GeglNode *start, GeglNode *end, const char *basepath,
                   {
                     const Babl *format;
                     const gchar *value = "";
-                    gegl_node_get (iter, properties[i]->name, &format, NULL);
+                    gegl_node_get (iter, property_name, &format, NULL);
                     if (format)
                       value = babl_get_name (format);
                     if (value[0] || (!trim_defaults))
@@ -1092,6 +1230,7 @@ gegl_node_new_from_serialized (const gchar *chaindata,
 {
   GeglNode *ret;
   GeglNode *foo;
+  gdouble time = 0.0;
 
   ret = gegl_node_new ();
   gegl_node_set (ret, "operation", "gegl:nop", NULL);
@@ -1099,7 +1238,7 @@ gegl_node_new_from_serialized (const gchar *chaindata,
   gegl_node_set (foo, "operation", "gegl:nop", NULL);
 
   gegl_node_link_many (foo, ret, NULL);
-  gegl_create_chain (chaindata, foo, ret, 0, 1024, path_root, NULL);
+  gegl_create_chain (chaindata, foo, ret, time, 1024, path_root, NULL);
 
   return ret;
 }
