@@ -37,7 +37,7 @@ property_double (tile_saturation, _("Tile saturation"), 2.5)
     value_range (0.0, 10.0)
 
 property_color  (bg_color, _("Background color"), "rgba(0.0, 0.0, 0.0, 0.0)")
-    description (("The tiles' background color"))
+    description (_("The tiles' background color"))
 
 property_seed (seed, _("Random seed"), rand)
 
@@ -619,7 +619,7 @@ gegl_op_class_init (GeglOpClass *klass)
     "title",       _("Cubism"),
     "name",        "gegl:cubism",
     "categories",  "artistic:scramble",
-    "reference-hash", "fe131f5ed2842b0b09739e16d7e5960d",
+    "reference-hash", "142b7257d4783a35afbbaaf185a1cf61",
     "license",     "GPL3+",
     "description", _("Convert the image into randomly rotated square blobs, somehow resembling a cubist painting style"),
     NULL);
