@@ -35,6 +35,7 @@ property_double (azimuth, _("Azimuth"), 30.0)
     description (_("Light angle (degrees)"))
     value_range (0, 360)
     ui_meta ("unit", "degree")
+    ui_meta ("direction", "ccw")
 
 property_double (elevation, _("Elevation"), 45.0)
     description (_("Elevation angle (degrees)"))
@@ -214,6 +215,7 @@ process (GeglOperation       *operation,
   const Babl    *format;
   gint           y;
   gint           floats_per_pixel;
+  float          factor = 1.0f / (1<<level);
 
   /*blur-map or emboss*/
   if (o->type == GEGL_EMBOSS_TYPE_BUMPMAP)
@@ -232,18 +234,26 @@ process (GeglOperation       *operation,
   rect.y      = result->y - op_area->top;
   rect.height = result->height + op_area->top + op_area->bottom;
 
+  if (level)
+  {
+    rect.x      *= factor;
+    rect.y      *= factor;
+    rect.width  *= factor;
+    rect.height *= factor;
+  }
+
   src_buf = g_new0 (gfloat, rect.width * rect.height * floats_per_pixel);
   dst_buf = g_new0 (gfloat, rect.width * rect.height * floats_per_pixel);
 
-  gegl_buffer_get (input, &rect, 1.0, format, src_buf,
+  gegl_buffer_get (input, &rect, factor, format, src_buf,
                    GEGL_AUTO_ROWSTRIDE, GEGL_ABYSS_NONE);
 
   /*do for every row*/
   for (y = 0; y < rect.height; y++)
     emboss (src_buf, &rect, dst_buf, &rect, o->type, y, floats_per_pixel,
-            DEG_TO_RAD (o->azimuth), DEG_TO_RAD (o->elevation), o->depth);
+            DEG_TO_RAD (o->azimuth), DEG_TO_RAD (o->elevation), o->depth * factor);
 
-  gegl_buffer_set (output, &rect, 0, format,
+  gegl_buffer_set (output, &rect, level, format,
                    dst_buf, GEGL_AUTO_ROWSTRIDE);
 
   g_free (src_buf);
@@ -268,7 +278,7 @@ gegl_op_class_init (GeglOpClass *klass)
   gegl_operation_class_set_keys (operation_class,
     "name",        "gegl:emboss",
     "title",       _("Emboss"),
-    "reference-hash", "ec49e055c3670e041c64b97b478b4667",
+    "reference-hash", "af0c6c39428853e1010fa4c51ee67c7d",
     "categories",  "light",
     "license",     "GPL3+",
     "description", _("Simulates an image created by embossing"),
