@@ -14,7 +14,7 @@
  *
  * Copyright 2006 Dominik Ernst <dernst@gmx.de>
  * Copyright 2013 Massimo Valentini <mvalentini@src.gnome.org>
- *           2017 Øyvind Kolås <pippin@gimp.org>
+ *     2017, 2018 Øyvind Kolås <pippin@gimp.org>
  *
  * Recursive Gauss IIR Filter as described by Young / van Vliet
  * in "Signal Processing 44 (1995) 139 - 151"
@@ -72,7 +72,6 @@ property_boolean (clip_extent, _("Clip to the input extent"), TRUE)
 
 #include "gegl-op.h"
 #include <math.h>
-#include <stdio.h>
 
 
 /**********************************************
@@ -88,6 +87,7 @@ typedef void (*IirYoungBlur1dFunc) (gfloat           *buf,
                                     const gfloat     *iminus,
                                     const gfloat     *uplus,
                                     const gint        len,
+                                    const gint        components,
                                     GeglAbyssPolicy   policy);
 
 static const gfloat white[4] = { 1.0f, 1.0f, 1.0f, 1.0f };
@@ -108,12 +108,13 @@ iir_young_find_constants (gfloat   sigma,
 
   const gdouble b0 = 1.57825 + q*(K1 + q*(    K2 + q *     K3));
   const gdouble b1 =           q*(K1 + q*(2 * K2 + q * 3 * K3));
-  const gdouble b2 =         - q*      q*(    K2 + q * 3 * K3);
+  const gdouble b2 =     (-K2 * q * q) + (-K3 * 3 * q * q * q);
   const gdouble b3 =           q*      q*          q *     K3;
 
   const gdouble a1 = b1 / b0;
   const gdouble a2 = b2 / b0;
   const gdouble a3 = b3 / b0;
+
   const gdouble c  = 1. / ((1+a1-a2+a3) * (1+a2+(a1-a3)*a3));
 
   m[0][0] = c * (-a3*(a1+a3)-a2 + 1);
@@ -201,6 +202,7 @@ iir_young_blur_1D_y (gfloat        *buf,
                      const gfloat  *iminus,
                      const gfloat  *uplus,
                      const gint     len,
+                     const gint     components,
                      GeglAbyssPolicy policy)
 {
   gint    i, j;
@@ -273,6 +275,7 @@ iir_young_blur_1D_yA (gfloat           *buf,
                       const gfloat     *iminus,
                       const gfloat     *uplus,
                       const gint        len,
+                      const gint        components,
                       GeglAbyssPolicy   policy)
 {
   gint    i, j;
@@ -322,6 +325,94 @@ iir_young_blur_1D_yA (gfloat           *buf,
     }
 }
 
+
+static inline void
+fix_right_boundary_generic (gdouble        *buf,
+                            gdouble       (*m)[3],
+                            const gfloat   *uplus,
+                            const gint      nc)
+{
+  gdouble u[nc*3];
+  gint    i, k, c;
+
+  for (k = 0; k < 3; k++)
+    for (c = 0; c < nc; c++)
+   u[k * nc + c] = buf[(-k-1)*nc+c] - uplus[c];
+
+  for (i = 0; i < 3; i++)
+    {
+      gdouble tmp[nc];
+      for (c = 0; c < nc ; c++)
+        tmp[c] = m[i][0] * u[0 * nc + c];
+
+      for (k = 1; k < 3; k++)
+        {
+          for (c = 0; c < nc ; c++)
+            tmp[c] += m[i][k] * u[k * nc + c];
+        }
+
+      for (c = 0; c < nc ; c++)
+        buf[nc * i + c] = tmp[c] + uplus[c];
+    }
+}
+
+static void
+iir_young_blur_1D_generic (gfloat           *buf,
+                           gdouble          *tmp,
+                           const gdouble    *b,
+                           gdouble         (*m)[3],
+                           const gfloat     *iminus,
+                           const gfloat     *uplus,
+                           const gint        len,
+                           const gint        components,
+                           GeglAbyssPolicy   policy)
+{
+  gint    i, j, c;
+  for (i = 0; i < 3; i++, tmp += components)
+    {
+      for (c = 0; c < components; c++)
+        tmp[c] = iminus[c];
+    }
+
+  buf += 3 * components;
+
+  for (i = 0; i < len; i++, buf += components, tmp += components)
+    {
+      for (c = 0; c < components; c++)
+        tmp[c] = b[0] * buf[c];
+
+      for (j = 1; j < 4; ++j)
+        {
+          gint offset = -components * j;
+
+          for (c = 0; c < components; c++)
+            tmp[c] += b[j] * tmp[offset + c];
+        }
+    }
+
+  fix_right_boundary_generic (tmp, m, uplus, components);
+
+  buf -= components;
+  tmp -= components;
+
+  for (i = 3 + len - 1; 3 <= i; i--, buf -= components, tmp -= components)
+    {
+      for (c = 0; c < components; c++)
+        tmp[c] *= b[0];
+
+      for (j = 1; j < 4; ++j)
+        {
+          gint offset = components * j;
+
+          for (c = 0; c < components; c++)
+            tmp[c] += b[j] * tmp[offset + c];
+        }
+
+      for (c = 0; c < components; c++)
+        buf[c] = tmp[c];
+    }
+}
+
 static inline void
 fix_right_boundary_rgb (gdouble        *buf,
                         gdouble       (*m)[3],
@@ -357,6 +448,7 @@ iir_young_blur_1D_rgb (gfloat           *buf,
                        const gfloat     *iminus,
                        const gfloat     *uplus,
                        const gint        len,
+                       const gint        components,
                        GeglAbyssPolicy   policy)
 {
   gint    i, j;
@@ -449,6 +541,7 @@ iir_young_blur_1D_rgbA (gfloat           *buf,
                         const gfloat     *iminus,
                         const gfloat     *uplus,
                         const gint        len,
+                        const gint        components,
                         GeglAbyssPolicy   policy)
 {
   gint     i, j;
@@ -540,7 +633,7 @@ iir_young_hor_blur (IirYoungBlur1dFunc   real_blur_1D,
                        GEGL_AUTO_ROWSTRIDE, GEGL_ABYSS_NONE);
 
       get_boundaries (policy, row, rect->width, nc, &iminus, &uplus);
-      real_blur_1D (row, tmp, b, m, iminus, uplus, rect->width, policy);
+      real_blur_1D (row, tmp, b, m, iminus, uplus, rect->width, nc, policy);
 
       gegl_buffer_set (dst, &cur_row, level, format, &row[3 * nc],
                        GEGL_AUTO_ROWSTRIDE);
@@ -580,7 +673,7 @@ iir_young_ver_blur (IirYoungBlur1dFunc   real_blur_1D,
                        GEGL_AUTO_ROWSTRIDE, GEGL_ABYSS_NONE);
 
       get_boundaries (policy, col, rect->height, nc, &iminus, &uplus);
-      real_blur_1D (col, tmp, b, m, iminus, uplus, rect->height, policy);
+      real_blur_1D (col, tmp, b, m, iminus, uplus, rect->height, nc, policy);
 
       gegl_buffer_set (dst, &cur_col, level, format, &col[3 * nc],
                        GEGL_AUTO_ROWSTRIDE);
@@ -914,7 +1007,8 @@ filter_disambiguation (GeglGblur1dFilter filter,
 {
   if (filter == GEGL_GBLUR_1D_AUTO)
     {
-      /* Threshold 1.0 is arbitrary */
+      /* Threshold 1.0 is arbitrary - but we really do not want IIR for much
+         smaller stdevs */
       if (std_dev < 1.0)
         filter = GEGL_GBLUR_1D_FIR;
       else
@@ -937,7 +1031,6 @@ gegl_gblur_1d_prepare (GeglOperation *operation)
   GeglProperties *o = GEGL_PROPERTIES (operation);
   const Babl *src_format = gegl_operation_get_source_format (operation, "input");
   const char *format     = "RaGaBaA float";
-
   o->user_data = iir_young_blur_1D_rgbA;
 
   /*
@@ -965,6 +1058,24 @@ gegl_gblur_1d_prepare (GeglOperation *operation)
           format = "YaA float";
           o->user_data = iir_young_blur_1D_yA;
         }
+      else if (babl_model_is (model, "cmyk"))
+        {
+          format = "cmyk float";
+          o->user_data = iir_young_blur_1D_generic;
+        }
+      else if (babl_model_is (model, "CMYK"))
+        {
+          format = "CMYK float";
+          o->user_data = iir_young_blur_1D_generic;
+        }
+      else if (babl_model_is (model, "cmykA") ||
+               babl_model_is (model, "camayakaA") ||
+               babl_model_is (model, "CMYKA") ||
+               babl_model_is (model, "CaMaYaKaA"))
+        {
+          format = "camayakaA float";
+          o->user_data = iir_young_blur_1D_generic;
+        }
     }
 
   gegl_operation_set_format (operation, "input", babl_format_with_space (format, space));
