@@ -11,7 +11,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with GEGL; if not, see <http://www.gnu.org/licenses/>.
+ * License along with GEGL; if not, see <https://www.gnu.org/licenses/>.
  *
  * Copyright 2017 Øyvind Kolås <pippin@gimp.org>
  */
@@ -63,7 +63,8 @@ property_double (angleboost, _("Angle Boost"), 0.0)
 property_double (twist, _("Black and green angle"), 75.0)
                  value_range (-180.0, 180.0)
                  ui_meta ("unit", "degree")
-                 description (_("angle offset for patterns"))
+                 ui_meta ("direction", "ccw")
+                 description (_("Angle offset for patterns"))
                  ui_meta ("label", "[color-model {white-on-black,"
                                    "              black-on-white} : bw-label,"
                                    " color-model {rgb}            : rgb-label,"
@@ -75,6 +76,7 @@ property_double (twist, _("Black and green angle"), 75.0)
 property_double (twist2, _("Red and cyan angle"), 15.0)
                  value_range (-180.0, 180.0)
                  ui_meta ("unit", "degree")
+                 ui_meta ("direction", "ccw")
                  ui_meta ("visible", "color-model {rgb, cmyk}")
                  ui_meta ("label", "[color-model {rgb}  : rgb-label,"
                                    " color-model {cmyk} : cmyk-label]")
@@ -84,6 +86,7 @@ property_double (twist2, _("Red and cyan angle"), 15.0)
 property_double (twist3, _("Blue and magenta angle"), 45.0)
                  value_range (-180.0, 180.0)
                  ui_meta ("unit", "degree")
+                 ui_meta ("direction", "ccw")
                  ui_meta ("visible", "color-model {rgb, cmyk}")
                  ui_meta ("label", "[color-model {rgb}  : rgb-label,"
                                    " color-model {cmyk} : cmyk-label]")
@@ -93,6 +96,7 @@ property_double (twist3, _("Blue and magenta angle"), 45.0)
 property_double (twist4, _("Yellow angle"), 0.0)
                  value_range (-180.0, 180.0)
                  ui_meta ("unit", "degree")
+                 ui_meta ("direction", "ccw")
                  ui_meta ("visible", "color-model {cmyk}")
 
 #else
@@ -103,13 +107,6 @@ property_double (twist4, _("Yellow angle"), 0.0)
 
 #include "gegl-op.h"
 
-static void
-prepare (GeglOperation *operation)
-{
-  gegl_operation_set_format (operation, "input", babl_format ("RGBA float"));
-  gegl_operation_set_format (operation, "output", babl_format ("RGBA float"));
-}
-
 #include <math.h>
 
 static inline float dmodf(float x, float y)
@@ -543,7 +540,6 @@ gegl_op_class_init (GeglOpClass *klass)
   point_filter_class = GEGL_OPERATION_POINT_FILTER_CLASS (klass);
 
   point_filter_class->process = process;
-  operation_class->prepare = prepare;
 #if 0
   point_filter_class->cl_process = cl_process;
   operation_class->opencl_support = TRUE;
